<?php

namespace App\Exports;

use Illuminate\Database\Eloquent\Collection;
use Maatwebsite\Excel\Concerns\Exportable;
use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\ToCollection;
use Maatwebsite\Excel\Concerns\WithChunkReading;
use Maatwebsite\Excel\Concerns\WithColumnFormatting;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\RegistersEventListeners;
use Maatwebsite\Excel\Concerns\WithValidation;
use Maatwebsite\Excel\Events\AfterSheet;
use PhpOffice\PhpSpreadsheet\Style\NumberFormat;
use PhpOffice\PhpSpreadsheet\Cell\DataValidation;


use PhpOffice\PhpSpreadsheet\Worksheet\Worksheet;
use PhpOffice\PhpSpreadsheet\Style\Font;



class GuestsTemplateExport implements FromCollection, WithEvents, WithColumnFormatting, WithHeadings, WithValidation,ShouldAutoSize
,WithChunkReading
{
    use Exportable, RegistersEventListeners;

    /**
     * @return \Illuminate\Support\Collection
     */
    public function collection()
    {
        $adriano = ['Adriano Dominick', '0768099975', 'DOUBLE', 300000, 300000,'Mr & Mrs'];
        $herbeth = ['Herbeth Norasco', '0678282112', 'SINGLE', 50000, 50000,'Mr'];
        $patrick = ['Patrick Nachenga', '0699444330', 'DOUBLE', 100000, 100000,'Mr & Mrs'];
        $othuman = ['Othuman Issa', '0626560698', 'SINGLE', 50000, 50000,'Ndg'];

        $template = collect([$adriano,$herbeth,$patrick,$othuman]);

        return $template;
    }

    public function headings(): array
    {
        return [
            'Name',
            'Phone',
            'Card Type',
            'Amount',
            'Paid Amount',
            'Title',
        ];
    }

    public function columnFormats(): array
    {
        return [
            'A' => '@',
            'B' => '@',
            'C' => '@',
            'D' => '@',
            'E' => '@',
            'F' => '@',
        ];
    }

    public function rules(): array
    {
        return [
            'C' => 'in:DOUBLE,SINGLE',
        ];
    }

    public function customValidationMessages()
    {
        return [
            'C.in' => 'Cell C must be either "DOUBLE" or "SINGLE".',
        ];
    }

    public function chunkSize(): int
    {
        return 100; // Adjust the chunk size as needed
    }

    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function (AfterSheet $event) {
                $event->sheet->getStyle('A1:F1')->applyFromArray([
                    'font' => [
                        'bold' => true,
                        'size' => 16,
                    ],

                ]);

                $event->sheet->getRowDimension(1)->setRowHeight(30);
                $event->sheet->getColumnDimension('A')->setWidth(30);
                $event->sheet->getColumnDimension('B')->setWidth(15);
                $event->sheet->getColumnDimension('D')->setWidth(15);
                $event->sheet->getColumnDimension('E')->setWidth(15);
                $rule = 'COUNTIF($B:$B, INDIRECT(ADDRESS(ROW(), COLUMN()))) = 1';

                // Apply the custom validation rule to all cells in column B
                $event->sheet->getDataValidation('B2:B1048576')->setFormula1($rule);
                $event->sheet->getDataValidation('B2:B1048576')->setType(DataValidation::TYPE_CUSTOM);
                $event->sheet->getDataValidation('B2:B1048576')->setErrorStyle(DataValidation::STYLE_INFORMATION);
                $event->sheet->getDataValidation('B2:B1048576')->setShowDropDown(false);
                $event->sheet->getDataValidation('B2:B1048576')->setErrorTitle('Duplicate Entry');
                $event->sheet->getDataValidation('B2:B1048576')->setError('This value is already in the column.');

                $validation = $event->sheet->getDataValidation('C2:C1048576'); // You can adjust the range as needed
                $validation->setType(DataValidation::TYPE_LIST);
                $validation->setErrorStyle(DataValidation::STYLE_STOP);
                $validation->setShowDropDown(true);
                $validation->setErrorTitle('Input error');
                $validation->setError('Please select a valid option from the dropdown list');
                $validation->setFormula1('"DOUBLE,SINGLE"');
                $event->sheet->getStyle('D')->getNumberFormat()->setFormatCode('_(* #,##0.00_);_(* (#,##0.00);_(* "-"??_);_(@_)'); // Currency format
                $event->sheet->getStyle('E')->getNumberFormat()->setFormatCode('_(* #,##0.00_);_(* (#,##0.00);_(* "-"??_);_(@_)'); //

            },

        ];
    }
}
