<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Pledges;
use App\Models\PledgepInvitations;
use SimpleSoftwareIO\QrCode\Facades\QrCode;
use DB;


class HomeController extends Controller
{
    //
    public function index(Request $request){
        return view('pages.index');
    }

    public function subscribers(Request $request){
        $search = $request->input('search.value') ;
        $length = $request->input('length') ;
        $start = $request->input('start');

        $query = Pledges::selectRaw('pledges.name,pledges.id, pledges.phone, pledges.amount, pledges.paid_amount')
        ->selectRaw(' pledge_invitations.qr_code,pledge_invitations.type,pledge_invitations.pledge_id, pledge_invitations.id as qr_id, pledge_invitations.check_in, pledge_invitations.code_number')
        ->join('pledge_invitations','pledge_invitations.pledge_id','=','pledges.id');

        $rows = [];
        if (!empty($search)) {
            $query->where(function($q) use ($search) {
            });
        }
        // $data = $query->offset($start)->limit($length)->get();
       $data = $query->get();
       $count = 1;
        foreach($data as $pledge){
             $rows[] = [
                $count,
                $pledge->name,
                $pledge->phone,
                $pledge->type,
                number_format((int)$pledge->amount,decimals:2).'/=',
                number_format((int)$pledge->paid_amount,decimals:2).'/=',
                $pledge->code_number,
                '<iframe src="'.url('generate-qr-code/'.$pledge->qr_id).'"></iframe>',

                '<button class="btn btn-primary editbutton">Download Card</button><input type="hidden" value="'.$pledge->id.'">'
             ];
             $count ++;
        }

        $totalRecords = $query->count();
        return response()->json([
             "recordsTotal" => $totalRecords,
             "recordsFiltered" => $totalRecords,
             "data" =>
                 $rows
         ]);
    }

    public function new_subscriber(Request $request){
        DB::beginTransaction();
        $name = $request->input('name');
        $amount = $request->input('amount') ?? 0;
        $paid_amount = $request->input('paid_amount') ?? 0;
        $phone = $request->input('phone');
        $type = $request->input('type') ?? 'SINGLE';
        $data =[
            'name' => $name,
            'amount' => $amount,
            'paid_amount' => $paid_amount,

        ];
        if($phone!=null){
            if(Pledges::where('phone',$phone)->first()){
                return Response()->json([
                    'status' => 'error',
                    'data' => [
                        'message' => 'Phone number is used'
                    ]
                ]);
            }
            $data += ['phone' =>$phone];

        }
        $pledges_id = Pledges::create($data)->id;
        $qr_code = crc32(uniqid()).$pledges_id.md5(time());
        $code = (string)$this->generate_unique_code().'-'.(string)$pledges_id;
        $data = [
            'pledge_id' => $pledges_id,
            'type' => $type,
            'qr_code' => $qr_code,
            'code_number' => $code
            // 'type' => 'SINGLE'
        ];
        PledgepInvitations::create($data);

        DB::commit();
        return Response()->json([
            'status' => 'success',
            'data' => [
                'message' => 'subscriber created'
            ]
        ]);

    }


    public function new_subscriber_from_excel($data){
        DB::beginTransaction();
        $name = $data['name'];
        $amount = $data['amount'] ?? 0;
        $paid_amount = $data['paid_amount'] ?? 0;
        $phone = $data['phone'];
        $type = $data['type'] ?? 'SINGLE';
        $data =[
            'name' => $name,
            'amount' => $amount,
            'paid_amount' => $paid_amount,

        ];

        if($phone!=null){
            if(Pledges::where('phone',$phone)->first()){
                // return [
                //     'status' => 'error',
                //     'data' => [
                //         'message' => 'Phone number is used'
                //     ]
                // ];
            }
            $data += ['phone' =>''];

        }
        $pledges_id = Pledges::create($data)->id;
        $qr_code = crc32(uniqid()).$pledges_id.md5(time());
         $code = (string)$this->generate_unique_code().'-'.(string)$pledges_id;
        $data = [
            'pledge_id' => $pledges_id,
            'type' => $type,
            'qr_code' => $qr_code,
            'code_number' => $code
            // 'type' => 'SINGLE'
        ];
        PledgepInvitations::create($data);

        DB::commit();
        return [
            'status' => 'success',
            'data' => [
                'message' => 'subscriber created'
            ]
        ];

    }

    public function verify_qr_code(Request $request){
       $qr_code = $request->input('qr_code');
       $query = PledgepInvitations::join('pledges as pledge','pledge_invitations.pledge_id','=','pledge.id')
       ->selectRaw('pledge.name, pledge_invitations.type, pledge.id,pledge_invitations.check_in ')
      // ->where('check_in','=',NULL)
       ->where('qr_code',$qr_code);
       $data = $query->first();
       if($data)
       {
         if($data->check_in){
            return response()->json(
            [
                'status' => 'failed',
                'data'    => 'Code expired'
            ]
        );
         }
       $query->update([
           'check_in' => now()
       ]);
        return response()->json(
            [
                'status' => 'success',
                'data'    => [
                    'name' => $data->name,
                    'type' => $data->type
                    ]
            ]
        );
       }
       else{
        return response()->json(
            [
                'status' => 'failed',
                'data'    => 'Unknown Guest'
            ]
        );
       }
    }

    public function api_get_subscribers(Request $request){
        $filter = $request->input('filter');
        $action = $request->input('action');
        $query = Pledges::selectRaw('pledges.name,pledges.id, pledges.phone, pledges.amount, pledges.paid_amount')
        ->selectRaw(' pledge_invitations.qr_code,pledge_invitations.type, pledge_invitations.check_in, pledge_invitations.code_number')
        ->join('pledge_invitations','pledge_invitations.pledge_id','=','pledges.id');
        if($filter){
            if($action == 'type'){
               $query->where('type',$filter);
            }
            if($action == 'check_in'){
               $query->where('check_in','!=','');
            }
          }
        return Response()->json($query->get());
    }

    public function generate_qr_code(Request $request, $id)
    {
        $data = PledgepInvitations::where('id',$id)->first();
        if($data){
        return QrCode::generate(
            $data->qr_code,
        );
     }
    }

    public function generate_unique_code($length = 4) {
        $characters = '0123456789';
        $code = '';
        for ($i = 0; $i < $length; $i++) {
            $randomIndex = rand(0, strlen($characters) - 1);
            $code .= $characters[$randomIndex];
        }
        return $code;
    }
}
